@@ -210,7 +210,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                              'http://www.sat.gob.mx/donat ' +
                              'http://www.sat.gob.mx/sitio_internet/cfd/donat/donat11.xsd')
             dom.setAttribute('xmlns:donat', 'http://www.sat.gob.mx/donat')
-        elif self.tipo_objeto == TRASLADO:
+        elif self.complemento == COMPLEMENTO_CARTA_PORTE:
             dom.setAttribute('xmlns:cfdi', 'http://www.sat.gob.mx/cfd/3')
             dom.setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
             dom.setAttribute('xmlns:cartaporte20', 'http://www.sat.gob.mx/CartaPorte20')
@@ -262,7 +262,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 for concepto in conceptos_list:
                     conceptos.appendChild(self.get_conceptos(concepto))
                 dom.appendChild(conceptos)
-            if self.data.get('iva') != '0' and self.data.get('iva') != 0 and self.data.get('tipo_comprobante') != 3:
+            if self.data.get('iva') != '0' and self.data.get('iva') != 0:
                 dom.appendChild(self.get_impuestos())
         elif self.tipo_objeto == PAGO:
             dom.setAttribute('SubTotal','0')
@@ -357,15 +357,15 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
             raise Exception("Ocurrio un error. Error::: " + str(e))
 
     def get_impuestos(self):
+        # Agrega seccion de impuesto total, falta TotalImpuestosRetenidos
         try:
             impuestos = self.dom.createElement('cfdi:Impuestos')
-            if self.data.get('tipo_comprobante') not in [ 5,6 ]:
-                if self.data.get('iva') != '0' and self.data.get('iva') != 0:
-                    impuestos.setAttribute('TotalImpuestosTrasladados',str(self.data.get('iva')))
-                    impuestos.appendChild(self.get_traslados())
-                else:
-                    impuestos.setAttribute('TotalImpuestosTrasladados','0')
-                    impuestos.appendChild(self.get_traslados())
+            if bool(self.data.get('aplica_retencion_iva', False)):
+                impuestos.setAttribute('TotalImpuestosRetenidos', str(self.data.get('retencion_iva')))
+                impuestos.appendChild(self.get_retenciones())
+            if bool(self.data.get('aplica_iva', False)):
+                impuestos.setAttribute('TotalImpuestosTrasladados',str(self.data.get('iva')))
+                impuestos.appendChild(self.get_traslados())
             return impuestos
         except Exception as e:
             raise Exception("Ocurrio un error. Error::: " + str(e))
@@ -373,24 +373,21 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
     def get_traslados(self):
         traslados = self.dom.createElement('cfdi:Traslados')
         traslado = self.dom.createElement('cfdi:Traslado')
-        traslado.setAttribute('Impuesto','002')
+        traslado.setAttribute('Impuesto', '002')
         if self.data.get('iva') != '0' and self.data.get('iva') != 0:
-            traslado.setAttribute('TipoFactor','Tasa')
-            traslado.setAttribute('TasaOCuota',str(self.data.get('porcentaje')))
-            traslado.setAttribute('Importe',str(self.data.get('iva')))
+            traslado.setAttribute('TipoFactor', self.data.get('tipo_factor', 'Tasa'))
+            traslado.setAttribute('TasaOCuota', str(self.data.get('porcentaje')))
+            traslado.setAttribute('Importe', str(self.data.get('iva')))
         else:
-            traslado.setAttribute('TipoFactor','Exento')
+            traslado.setAttribute('TipoFactor', 'Exento')
         traslados.appendChild(traslado)
         return traslados
 
     def get_retenciones(self):
         retenciones = self.dom.createElement('cfdi:Retenciones')
         retencion = self.dom.createElement('cfdi:Retencion')
-        retencion.setAttribute('Impuesto','002')
-        if self.data.get('iva') != '0' and self.data.get('iva') != 0:
-            retencion.setAttribute('Importe',str(self.data.get('iva')))
-        else:
-            retencion.setAttribute('Importe','0')
+        retencion.setAttribute('Impuesto', '002')
+        retencion.setAttribute('Importe', str(self.data.get('retencion_iva')))
         retenciones.appendChild(retencion)
         return retenciones
 
@@ -499,7 +496,8 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 raise Exception('Se debe especificar una descripción para poder timbrar y posteriormente crear una poliza.')
             cfdi_concepto.setAttribute('ValorUnitario', str(concepto.get('valor_unitario')))
             cfdi_concepto.setAttribute('Importe', str(concepto.get('importe')))
-            if self.tipo_objeto != TRASLADO and self.complemento != COMPLEMENTO_CARTA_PORTE:
+            if self.tipo_objeto != TRASLADO:
+                #revisar en el manual si lleva impuestos este tipo de comprobante
                 cfdi_concepto.appendChild(self.get_impuestos_concepto(concepto))
             if concepto.get('informacion_aduanera'):
                 cfdi_concepto.appendChild(self.get_informacion_aduanera(concepto.get('informacion_aduanera')))
@@ -511,26 +509,37 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
 
     def get_impuestos_concepto(self,concepto):
         impuestos = self.dom.createElement('cfdi:Impuestos')
-        impuestos.appendChild(self.get_traslados_retencion_concepto(concepto))
+        if bool(self.data.get('aplica_iva', False)):
+            impuestos.appendChild(self.get_traslados_concepto(concepto))
+        if bool(self.data.get('aplica_retencion_iva', False)):
+            impuestos.appendChild(self.get_retenciones_concepto(concepto))
         return impuestos
 
-    def get_traslados_retencion_concepto(self, concepto, retenciones = False):
-        trasretcons = self.dom.createElement('cfdi:Traslados')
-        trasretcon = self.dom.createElement('cfdi:Traslado')
-        if retenciones:
-            trasretcons = self.dom.createElement('cfdi:Retenciones')
-            trasretcon = self.dom.createElement('cfdi:Retencion')
-        trasretcon.setAttribute('Impuesto','002')  # 'IVA'
+    def get_traslados_concepto(self, concepto):
+        traslados = self.dom.createElement('cfdi:Traslados')
+        traslado = self.dom.createElement('cfdi:Traslado')
+        traslado.setAttribute('Impuesto','002')  # 'IVA'
         if concepto.get('iva') != '0' and concepto.get('iva') != 0:
-            trasretcon.setAttribute('TipoFactor','Tasa')
-            trasretcon.setAttribute('Base', str(concepto.get('base')))
-            trasretcon.setAttribute('TasaOCuota', str(concepto.get('porcentaje')))
-            trasretcon.setAttribute('Importe', str(concepto.get('iva')))
+            traslado.setAttribute('TipoFactor', concepto.get('tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
+            traslado.setAttribute('Base', str(concepto.get('base')))  # Importe del concepto
+            traslado.setAttribute('TasaOCuota', str(concepto.get('porcentaje')))  # valor del impuesto que se envia
+            traslado.setAttribute('Importe', str(concepto.get('iva')))
         else:
-            trasretcon.setAttribute('TipoFactor','Exento')
-            trasretcon.setAttribute('Base', str(concepto.get('importe')))
-        trasretcons.appendChild(trasretcon)
-        return trasretcons
+            traslado.setAttribute('TipoFactor', 'Exento')
+            traslado.setAttribute('Base', str(concepto.get('importe')))
+        traslados.appendChild(traslado)
+        return traslados
+
+    def get_retenciones_concepto(self, concepto):
+        retenciones = self.dom.createElement('cfdi:Retenciones')
+        retencion = self.dom.createElement('cfdi:Retencion')
+        retencion.setAttribute('Impuesto', '002')  # 'IVA'
+        retencion.setAttribute('TipoFactor', concepto.get('retencion_tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
+        retencion.setAttribute('Base', str(concepto.get('base')))  # Importe del concepto
+        retencion.setAttribute('TasaOCuota', str(concepto.get('retencion_porcentaje')))  # valor del impuesto que se envia
+        retencion.setAttribute('Importe', str(concepto.get('retencion_iva')))
+        retenciones.appendChild(retencion)
+        return retenciones
 
     def get_informacion_aduanera(self, informacion_aduanera):
         try:
